#!/usr/bin/perl

package BACnet;
use strict;
use warnings;

our $VERSION = '0.08';

1;

=encoding UTF-8

=head1 NAME

BACnet – Placeholder namespace for the BACnet Perl toolkit

=head1 DESCRIPTION

This namespace serves only as an entry point.  
The actual implementation is located in:

L<BACnet::Device|https://metacpan.org/pod/BACnet::Device>

Please refer directly to that module for full documentation, usage examples,
and API details.

=cut
